-- Ejemplo de Sort Warnings en SQL Server
-- provocado por la existencia de Estadsticas NO actualizadas
--
--
-- GuilleSQL - Un portal sobre Microsoft SQL Server en Castellano
-- http://www.guillesql.es
--
USE AdventureWorks
GO


-- ***  Creamos un ndice adicional NO AGRUPADO para este Laboratorio  ***
CREATE NONCLUSTERED INDEX IX_SalesOrderDetail_SalesOrderID_SalesOrderDetailID
ON [Sales].[SalesOrderDetail] ([ModifiedDate])
INCLUDE ([SalesOrderID],[SalesOrderDetailID])
GO


-- ***  Creamos un Procedure para utilizar en el ejemplo  ***
CREATE PROCEDURE dbo.SalesOrderSelect 
   @ModifiedDateFrom datetime
   ,@ModifiedDateTo datetime 
AS
BEGIN

   SELECT so.SalesOrderID, so.ModifiedDate 
   FROM  Sales.SalesOrderDetail so 
   WHERE so.ModifiedDate BETWEEN @ModifiedDateFrom AND @ModifiedDateTo 
   ORDER BY so.SalesOrderDetailID 
   OPTION (MAXDOP 1)

END
GO

-- ***  Habilitamos las estadsticas de IO                                    ***
-- ***  Nos permitir conocer si el Optimizador de Consultas crea WorkTables  ***
SET STATISTICS IO ON;


-- ***  Comprobar el acceso a TEMPDB usando las DMVs  ***
SELECT 
   database_id, file_id
   , num_of_reads, num_of_bytes_read
   , num_of_writes, num_of_bytes_written
FROM sys.dm_io_virtual_file_stats(NULL, NULL)
WHERE database_id=DB_ID('TEMPDB')


-- ***  Invocacin inicial al Procedimiento Almacenado, con las Estadsticas OK  ***
exec dbo.SalesOrderSelect '2001-01-01 00:00:00.000', '2002-12-31 00:00:00.000' 

-- ***  Falsear Estadsticas y
-- ***  forzar la generacin de un nuevo Plan de Ejecucin
UPDATE STATISTICS Sales.SalesOrderDetail
WITH ROWCOUNT  = 500
GO
EXEC sp_recompile 'SalesOrderSelect'
GO

-- ***  Invocacin al Procedimiento Almacenado, con las Estadsticas falseadas  ***
exec dbo.SalesOrderSelect '2001-01-01 00:00:00.000', '2002-12-31 00:00:00.000' 


-- ***  Deshabilitamos las estadsticas de IO ***
SET STATISTICS IO OFF;

-- ***  Eliminamos el Procedure utilizado en el ejemplo  ***
DROP PROCEDURE dbo.SalesOrderSelect

-- ***  Eliminamos el ndice NO AGRUPADO creado para este Laboratorio  ***
DROP INDEX [Sales].[SalesOrderDetail].IX_SalesOrderDetail_SalesOrderID_SalesOrderDetailID
GO